@@ -14,18 +14,24 @@
 #include <linux/mutex.h>
 #include <linux/net.h>
 #include <linux/padata.h>
-#include <linux/notifier.h>
+
+struct wireguard_device;
+struct handshake_worker {
+	struct wireguard_device *wg;
+	struct work_struct work;
+};
 
 struct wireguard_device {
+	struct list_head device_list;
 	struct sock __rcu *sock4, *sock6;
 	u16 incoming_port;
+	u32 fwmark;
 	struct net *creating_net;
-	struct workqueue_struct *workqueue;
-	struct workqueue_struct *parallelqueue;
-	struct padata_instance *parallel_send, *parallel_receive;
 	struct noise_static_identity static_identity;
+	struct workqueue_struct *incoming_handshake_wq, *peer_wq;
 	struct sk_buff_head incoming_handshakes;
-	struct work_struct incoming_handshakes_work;
+	atomic_t incoming_handshake_seqnr;
+	struct handshake_worker __percpu *incoming_handshakes_worker;
 	struct cookie_checker cookie_checker;
 	struct pubkey_hashtable peer_hashtable;
 	struct index_hashtable index_hashtable;
@@ -33,8 +39,9 @@ struct wireguard_device {
 	struct list_head peer_list;
 	struct mutex device_update_lock;
 	struct mutex socket_update_lock;
-#ifdef CONFIG_PM_SLEEP
-	struct notifier_block clear_peers_on_suspend;
+#ifdef CONFIG_WIREGUARD_PARALLEL
+	struct workqueue_struct *crypt_wq;
+	struct padata_instance *encrypt_pd, *decrypt_pd;
 #endif
 };
 
